/*
 * imagenbbdd.java
 *
 * @author  Roberto Canales
 * Ejemplo de manipulacin de imagenes en Base de Datos
 *
 */

import java.io.*;
import javax.sql.*;
import java.sql.*;
import java.awt.*;

public class CImagenbbdd 
{
    
    public static void main(String[] args) 
    {
        CImagenbbdd nuevo = new CImagenbbdd();
        nuevo.ejecuta();
    }
    
    // mostramos un mensaje de depuracin del sistema
    public void depura(String param) 
    {
        System.out.println("Mensaje depuracin: " + param);
    }
    
    // ejecutamos el proceso
    public void ejecuta() 
    {
        long inicio = System.currentTimeMillis();

        Connection con = getConexion();

        long fin = System.currentTimeMillis() - inicio;
        depura("El tiempo transcurrido en conectar es " + fin + " milisegundos");
        
        InputStream imagenRecuperada = recuperaImagenBBDD(con);
        guardaImagenEnDisco(imagenRecuperada);
        
        fin = System.currentTimeMillis() - inicio;
        depura("El tiempo transcurrido completo es " + fin + " milisegundos");

        escribeImagenEnBBDD(con);
        
        fin = System.currentTimeMillis() - inicio;
        depura("El tiempo transcurrido completo es " + fin + " milisegundos");
    }
    
    // guardamos desde el stream de base de datos a disco
    public void guardaImagenEnDisco(InputStream imagenBuffer) 
    {
        try
        {
            File fichero = new File("c:\\melones.jpg");        
            BufferedInputStream in = new BufferedInputStream(imagenBuffer);        
            BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(fichero));        

            byte[] bytes = new byte[8096];       
            int len = 0;        

            depura("Copiamos el fichero");
            
            while ( (len = in.read( bytes )) > 0 )
            {           
                out.write( bytes, 0, len );        
            }       

            out.flush();        
            out.close();        
            in.close();
            
            depura("Teminacin del proceso con xito");
        }
        catch(Exception e)
        {
            depura("Error al escribir en disco " + e.getMessage());
        }
    }
    
    // recuperamos la conexion
    public Connection getConexion() 
    {
        Connection con = null;
        
        try 
        {
            Driver d = (Driver)Class.forName("com.mysql.jdbc.Driver").newInstance();
            
            depura("Recuperamos conexin");
            con = DriverManager.getConnection("jdbc:mysql://localhost/tutoriales","","");
        }
        catch(Exception e) 
        {
            depura("Error al recuperar conexion " + e.toString());
            return null;
        }
        
        return con;
    }

    
    // leemos de la base de datos
    public InputStream recuperaImagenBBDD(Connection con) 
    {
        try 
        {
            String consultaGenerada = "select * from imagenesbbdd limit 1";
            
            depura("Ejecutamos Statment");
            java.sql.Statement stmt = con.createStatement();
            
            depura("Ejecutamos sentencia " + consultaGenerada);
            ResultSet results = stmt.executeQuery(consultaGenerada);
            
            results.next();
            
            Blob campo = results.getBlob("imagen");
            return campo.getBinaryStream();
        }
        catch(Exception e) 
        {
            depura("Error al recuperar el Stream");
        }
        
        return null;
    }
    
    // escribimos la imagen en la base de datos
    void escribeImagenEnBBDD(Connection con)
    {
        try 
        {
              File fichero = new File("c:\\melones.jpg");  // no es muy elegante pero esto es solo un ejemplo
              FileInputStream streamEntrada = new FileInputStream(fichero);
              PreparedStatement pstmt = con.prepareStatement("insert into imagenesbbdd (nombre,imagen) values (?,?)",Statement.RETURN_GENERATED_KEYS);
              pstmt.setString(1, "melones.jpg");
              pstmt.setBinaryStream(2, streamEntrada, (int)fichero.length());
              pstmt.executeUpdate();
              
              // recuperamos el indice del elemento generado
              ResultSet rs = pstmt.getGeneratedKeys();
              rs.next();
              
              depura("El id recuperado es: " + rs.getInt(1));
              
              rs.close();
              pstmt.close();
              streamEntrada.close();

        }
        catch(Exception e) 
        {
            depura("Error al escribir el Stream " + e.getMessage());
        }
    }
}
